#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#  Name:	OneShot
#    By:	sdmitch
# Usage:	Creates a grouped 3d Box
#  Date:	Sep 2011
#------------------------------------------------------------------------------------------------
require 'sketchup'
#----------------------------------------------------------------------------
if( not file_loaded?("OneShot.rb") )
    UI.menu("Plugins").add_item("One Shot") { Sketchup.active_model.select_tool OneShot.new }
end
#-----------------------------------------------------------------------------
file_loaded("OneShot.rb") # load"OneShot.rb"

class OneShot

	def initialize
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@aMod = Sketchup.active_model
		@aEnt = @aMod.active_entities
		@aSel = @aMod.selection
		self.reset
	end

	def reset
		@state = 0
		@ip1.clear
		@aSel.clear
		@drawn = false
		Sketchup::set_status_text("", SB_VCB_LABEL)
		Sketchup::set_status_text("", SB_VCB_VALUE)
	end

	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		self.box if @state==1
		self.cube if @state==2
		view.lock_inference
		Sketchup::set_status_text("Pick First Corner", SB_PROMPT) if @state==0
		Sketchup::set_status_text("Pick Second Corner", SB_PROMPT) if @state==1
		Sketchup::set_status_text("Define Height", SB_PROMPT) if @state==2
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		@ip1.copy! @ip
		@pt3 = @ip.position
		view.tooltip = @ip.tooltip     
		view.lock_inference
		view.invalidate
	end

	def onLButtonDown(flags, x, y, view)
		self.set_current_point(x, y, view)
		self.increment_state
		view.lock_inference
	end
	
	def increment_state
		@state += 1
		case @state
			when 1
				@pt1 = @ip.position
				# Sketchup.send_action "viewTop:"
			when 2	
				@pt2 = @ip.position
				# Sketchup.send_action "viewIso:"
			when 3
				@dist=@pt3.distance_to_plane @plane
				self.make_grp
				self.reset
		end
	end
	
	def box
		xlo = [@pt1.x,@pt3.x].min
		ylo = [@pt1.y,@pt3.y].min
		xhi = [@pt1.x,@pt3.x].max
		yhi = [@pt1.y,@pt3.y].max
		@bllc=[xlo,ylo,@pt1.z]
		@bulc=[xlo,yhi,@pt1.z]
		@burc=[xhi,yhi,@pt1.z]
		@blrc=[xhi,ylo,@pt1.z]
		@plane=Geom.fit_plane_to_points @bllc,@bulc,@burc,@blrc
		Sketchup::set_status_text("Dimensions:", SB_VCB_LABEL)
		Sketchup::set_status_text("#{(xhi-xlo).to_l},#{(yhi-ylo).to_l}", SB_VCB_VALUE)
	end
	
	def cube
		@tllc=[@bllc.x,@bllc.y,@pt3.z]
		@tulc=[@bulc.x,@bulc.y,@pt3.z]
		@turc=[@burc.x,@burc.y,@pt3.z]
		@tlrc=[@blrc.x,@blrc.y,@pt3.z]
		@dist=@pt3.distance_to_plane @plane
		Sketchup::set_status_text("Height:", SB_VCB_LABEL)
		Sketchup::set_status_text(@dist.to_l, SB_VCB_VALUE)
	end
	
	def make_grp
		@aMod.start_operation "One Shot"
		grp=@aEnt.add_group;ge=grp.entities
		face=ge.add_face(@bllc,@bulc,@burc,@blrc)
		face.pushpull -@dist
		@aMod.commit_operation
	end
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end
	
	def onRButtonDown(flags, x, y, view)
		Sketchup.send_action("selectSelectionTool:") if @state==0
		self.reset
	end

	def draw(view)
		@drawn = false
		# Show the current input point
		if( @ip.valid? && @ip.display? ) then
			@ip.draw(view)
			@drawn = true
		end
		if @state > 0
			view.drawing_color = "magenta"
			view.line_width = 3 #if inference_locked
			if( @state == 1 )
				view.draw(GL_LINE_LOOP, @bllc,@bulc,@burc,@blrc)
				view.invalidate
			else
				view.draw(GL_LINE_LOOP, @bllc,@bulc,@burc,@blrc)
				view.draw(GL_LINE_LOOP, @tllc,@tulc,@turc,@tlrc)
				view.draw(GL_LINES,@bllc,@tllc,@bulc,@tulc,@burc,@turc,@blrc,@tlrc)
				view.invalidate
			end
			view.line_width = 1 #if inference_locked
			@drawn = true
		end
		view.lock_inference
	end
end # of class OneShot
